/*
Copyright 2022, Savanni D'Gerinel <savanni@luminescent-dreams.com>

This file is part of Savanni's AVR library.

Lumeto is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Lumeto is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Lumeto. If not, see <https://www.gnu.org/licenses/>.
*/

#include <util/delay.h>
#include <dio.h>
#include "shift_register.h"

void sr_strobe_line(dio_t *line) {
    dio_set(line, 1);
    _delay_us(1);
    dio_set(line, 0);
}

void sr_initialize(shift_register_t *reg) {
    dio_set_direction(&reg->output, LINE_OUT);
    dio_set_direction(&reg->shift_clock, LINE_OUT);
    dio_set_direction(&reg->latch_clock, LINE_OUT);

    dio_set(&reg->output, 0);
    dio_set(&reg->shift_clock, 0);
    dio_set(&reg->latch_clock, 0);
}

void sr_send_lsb(shift_register_t *reg, uint8_t byte) {
    for (uint8_t i = 0; i < 8; i++) {
        byte & _BV(i) ? dio_set(&reg->output, 1) : dio_set(&reg->output, 0);
        sr_strobe_line(&reg->shift_clock);
    }
    dio_set(&reg->output, 0);
    sr_strobe_line(&reg->latch_clock);
}

void sr_send_msb(shift_register_t *reg, uint8_t byte) {
    for (int8_t i = 7; i >= 0; i--) {
        byte & _BV(i) ? dio_set(&reg->output, 1) : dio_set(&reg->output, 0);
        sr_strobe_line(&reg->shift_clock);
    }
    dio_set(&reg->output, 0);
    sr_strobe_line(&reg->latch_clock);
}


