/*
Copyright 2022, Savanni D'Gerinel <savanni@luminescent-dreams.com>

This file is part of Savanni's AVR library.

Lumeto is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Lumeto is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Lumeto. If not, see <https://www.gnu.org/licenses/>.
*/

#include "sk9822.h"

void send_byte(dio_t data_pin, dio_t clock_pin, uint8_t byte) {
    for (int i = 7; i >= 0; i--) {
        dio_set(&data_pin, byte & _BV(i));

        dio_set(&clock_pin, 1);
        dio_set(&clock_pin, 0);
    }
}

void send_start(dio_t data_pin, dio_t clock_pin) {
    dio_set(&clock_pin, 0);
    send_byte(data_pin, clock_pin, 0);
    send_byte(data_pin, clock_pin, 0);
    send_byte(data_pin, clock_pin, 0);
    send_byte(data_pin, clock_pin, 0);
}

void send_term(dio_t data_pin, dio_t clock_pin) {
    send_byte(data_pin, clock_pin, 0xff);
    send_byte(data_pin, clock_pin, 0xff);
    send_byte(data_pin, clock_pin, 0xff);
    send_byte(data_pin, clock_pin, 0xff);
    dio_set(&clock_pin, 1);
}

void sk9822_init(sk9822_t *lights) {
    dio_set_direction(&lights->data_pin, LINE_OUT);
    dio_set_direction(&lights->clock_pin, LINE_OUT);
}

void sk9822_send(sk9822_t *lights, rgb_t * pixels, uint8_t count) {
    send_start(lights->data_pin, lights->clock_pin);
    for (uint8_t i = 0; i < count; i++) {
        send_byte(lights->data_pin, lights->clock_pin, 0xe0 + pixels[i].brightness);
        send_byte(lights->data_pin, lights->clock_pin, pixels[i].r);
        send_byte(lights->data_pin, lights->clock_pin, pixels[i].b);
        send_byte(lights->data_pin, lights->clock_pin, pixels[i].g);
    }
    send_term(lights->data_pin, lights->clock_pin);
}
