/*
Copyright 2022, Savanni D'Gerinel <savanni@luminescent-dreams.com>

This file is part of Savanni's AVR library.

Lumeto is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Lumeto is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Lumeto. If not, see <https://www.gnu.org/licenses/>.
*/


#include <util/delay.h>
#include "spi.h"

void spi_initialize(spi_t *spi) {
    dio_set_direction(&spi->clock, LINE_OUT);
    dio_set_direction(&spi->data_out, LINE_OUT);
    dio_set_direction(&spi->data_in, LINE_IN);
    dio_set_direction(&spi->chip_select, LINE_OUT);

    dio_set(&spi->chip_select, 1);
}

void spi_acquire(spi_t *spi) {
    dio_set(&spi->chip_select, 0);
}

void spi_release(spi_t *spi) {
    dio_set(&spi->chip_select, 1);
}

uint8_t spi_transceive(spi_t *spi, uint8_t output) {
    uint8_t input = 0;
    int input_bit;

    for (int i = 7; i >= 0; i--) {
        dio_set(&spi->clock, 0);
        if (output & _BV(i)) {
            dio_set(&spi->data_out, 1);
        } else {
            dio_set(&spi->data_out, 0);
        }
        _delay_us(10);
        dio_set(&spi->clock, 1);
        input_bit = dio_read(&spi->data_in);
        input = (input << 1) | input_bit;
        _delay_us(10);
    }
    dio_set(&spi->clock, 0);
    _delay_us(10);
    return input;
}
